<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Platforms\SQLAnywhere11Platform;
use Doctrine\ORM\EntityRepository;
use Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Resource extends EntityRepository {
    
	public function getResources($organizationId = 0, $internalConditions="", $externalConditions = ""){
    	$query = <<<SQL
    	SELECT *
    	  FROM (
        	SELECT DISTINCT R.ID, R.NAME, R.RESOURCES_TYPE, R.STATUS_RESOURCE, 
        	       R.EXTERNAL_ID, (TO_CHAR(R.AVAILABLE_FROM, 'DD/MM/YYYY')) AS DISPONIBILITY
              FROM (
          		SELECT R.ID, R.NAME, R.RESOURCES_TYPE, R.STATUS AS STATUS_RESOURCE,
          		       R.EXTERNAL_ID, R.AVAILABLE_FROM, R.CLASS
          		  FROM RESOURCE_T R
          		 WHERE R.ORGANIZATION = :organization) R
          LEFT JOIN PERMISSION PER ON(R.ID = PER.RESOURCE_T )
          	  WHERE (R.STATUS_RESOURCE <> 'D')
          		    AND R.CLASS <> 'S'
          		    AND (R.RESOURCES_TYPE = 'W' OR R.RESOURCES_TYPE = 'A' OR R.RESOURCES_TYPE = 'D')
          		    $internalConditions
          		    )
          WHERE 1=1
                $externalConditions
       ORDER BY NAME ASC
SQL;

        $param = array(
			'organization' => $organizationId
		);
        
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function getAclProfileIds($resourceId){
    	$query = <<<SQL
    	  SELECT PERF.NRPERFILACESSO AS "ID"
            FROM RESOURCE_T RES
            JOIN OPERADOR OPE ON(OPE.CDOPERADOR = RES.EXTERNAL_CODE)
            JOIN PERFACESSPARC PEPA ON(OPE.NRPARCNEGOCIO = PEPA.NRPARCNEGOCIO)
            JOIN PERFILACESSO PERF ON (PERF.NRPERFILACESSO = PEPA.NRPERFILACESSO AND PERF.NRORG = PEPA.NRORG)
           WHERE RES.ID = :RESOURCE_ID
SQL;

        $param = array(
			'RESOURCE_ID' => $resourceId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function  allResourcesOrganization($organizationId, $filterString = ''){

		$query = <<<SQL
		SELECT * FROM (
SELECT DISTINCT R.ID,R.NAME,(CASE
					WHEN (R.RESOURCES_TYPE = 'W') THEN 'Trabalho'
					WHEN (R.RESOURCES_TYPE = 'A') THEN 'Ativo'
					ELSE ''
				END) AS TYPE
        FROM(
      SELECT R.ID,R.NAME,R.RESOURCES_TYPE,R.ORGANIZATION,(
      SELECT
      MAX(T.ESTIMATED_END_DATE) AS UNOCCUPANCY_DATE
        FROM 		TASK T
        INNER JOIN 	PROJECT PR 			ON (T.PROJECT = PR.ID)
        INNER JOIN 	ALLOCATION AL 		ON (AL.TASK = T.ID)
          WHERE
          AL.RESOURCE_T = R.ID
          AND 	(PR.STATUS = 'A' OR PR.STATUS IS NULL)
          AND 	(T.STATUS = 'A' OR T.STATUS IS NULL)
          AND (T.PERCENT_COMPLETED < 100 OR T.PERCENT_COMPLETED IS NULL)
          GROUP BY AL.RESOURCE_T) AS UNOCCUPANCY_DATE,
          (
            SELECT
            count(1) AS UNOCCUPANCY_DATE
              FROM 		TASK T
              INNER JOIN 	PROJECT PR 			ON (T.PROJECT = PR.ID)
              INNER JOIN 	ALLOCATION AL 		ON (AL.TASK = T.ID)
                WHERE
                AL.RESOURCE_T = R.ID
                AND 	(PR.STATUS = 'A' OR PR.STATUS IS NULL)
                AND 	(T.STATUS = 'A' OR T.STATUS IS NULL)
                AND (T.PERCENT_COMPLETED < 100 OR T.PERCENT_COMPLETED IS NULL)
                GROUP BY AL.RESOURCE_T
            ) AS QUEUED_ACTIVITIES,
            (
            SELECT
            count(1) AS UNOCCUPANCY_DATE
              FROM 		TASK T
              INNER JOIN 	PROJECT PR 			ON (T.PROJECT = PR.ID)
              INNER JOIN 	ALLOCATION AL 		ON (AL.TASK = T.ID)
                WHERE
                AL.RESOURCE_T = R.ID
                AND 	(PR.STATUS = 'A' OR PR.STATUS IS NULL)
                AND 	(T.STATUS = 'A' OR T.STATUS IS NULL)
                AND (T.PERCENT_COMPLETED < 100 OR T.PERCENT_COMPLETED IS NULL)
                GROUP BY AL.RESOURCE_T
            ) AS STATUS,R.CLASS
      FROM RESOURCE_T R) R
      WHERE R.ORGANIZATION = :organizationId
      AND R.CLASS <> 'S'
      AND (R.RESOURCES_TYPE = 'W' OR R.RESOURCES_TYPE = 'A')
      )
      WHERE 1 = 1
      $filterString
      ORDER BY NAME
SQL;
		$param = array(
			'organizationId' => $organizationId
		);
		

		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

    public function findNameOperador($cdoperador){

        $query = <<<SQL
        SELECT NMOPERADOR FROM OPERADOR
        WHERE CDOPERADOR = :cdoperador
SQL;
        $param = array(
            ':cdoperador' => $cdoperador
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findSponsorIdByCdoperador($cdoperador){

        $query = <<<SQL
    SELECT RES.ID
        FROM OPERADOR OPE
        JOIN RESOURCE_T RES ON(RES.EXTERNAL_CODE = OPE.CDCLIENTE)
        WHERE OPE.CDOPERADOR = :cdoperador
SQL;
        $param = array(
            ':cdoperador' => $cdoperador
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findAccessProfile($organization,$filterSring){

        $query = <<<SQL
    SELECT *
      FROM PERFILACESSO
      WHERE IDATIVO = 'S'
        $filterSring
        ORDER BY NMPERFILACESSO
SQL;
        $param = array(
            ':organization' => $organization
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findAccessProfileOfResource($resourceId,$filterSring = ''){

        $query = <<<SQL
        SELECT * FROM (
              SELECT OPE.CDOPERADOR,
                     RES.ID AS RESOURCE_T,
                     PER.NRPERFACESSPARC AS ID,PEF.NMPERFILACESSO AS PROFILE_NAME, PEF.NRPERFILACESSO
                FROM PERFACESSPARC PER
                JOIN OPERADOR OPE ON(PER.NRPARCNEGOCIO = OPE.NRPARCNEGOCIO AND PER.NRORG = OPE.NRORG)
                JOIN RESOURCE_T RES ON (RES.EXTERNAL_CODE = OPE.CDOPERADOR)
                JOIN PERFILACESSO PEF ON (PEF.NRPERFILACESSO = PER.NRPERFILACESSO)
               WHERE  PEF.IDATIVO = 'S' AND RES.ID = :resourceId)
               WHERE 1 = 1
               $filterSring
SQL;
        $param = array(
            ':resourceId' => $resourceId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function getNrparcnegocioByCdoperador($resourceId){

        $query = <<<SQL
        SELECT OPE.NRPARCNEGOCIO AS ID
          FROM RESOURCE_T RES
          JOIN OPERADOR OPE ON (RES.EXTERNAL_CODE = OPE.CDOPERADOR)
         WHERE RES.ID = :resourceId
SQL;
        $param = array(
            ':resourceId' => $resourceId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return empty($result) ? null : $result[0]['ID'];
    }

    public function  ResourcesOrganization($organizationId, $resourceID){

		$query = <<<SQL
SELECT DISTINCT R.ID,R.NAME,R.RESOURCES_TYPE,R.STATUS_RESOURCE,
				(CASE
					WHEN (QUEUED_ACTIVITIES = 0 OR QUEUED_ACTIVITIES IS NULL) THEN 'Disponível'
					ELSE 'Em atividade'
				END) AS STATUS,
				(CASE
					WHEN (QUEUED_ACTIVITIES = 0 OR QUEUED_ACTIVITIES IS NULL) THEN 'Agora'
					ELSE TO_CHAR(UNOCCUPANCY_DATE,'DD/MM/YYYY')
				END) AS DISPONIBILITY,
				(CASE
					WHEN QUEUED_ACTIVITIES IS NULL THEN 0
					ELSE QUEUED_ACTIVITIES
				END) AS QUEUED_ACTIVITIES
        FROM(
  SELECT R.ID,R.NAME,R.RESOURCES_TYPE,R.STATUS AS STATUS_RESOURCE,R.ORGANIZATION,(
  SELECT
  MAX(T.ESTIMATED_END_DATE) AS UNOCCUPANCY_DATE
    FROM 		TASK T
    INNER JOIN 	PROJECT PR 			ON (T.PROJECT = PR.ID)
    INNER JOIN 	ALLOCATION AL 		ON (AL.TASK = T.ID)
      WHERE
      AL.RESOURCE_T = R.ID
      AND 	(PR.STATUS = 'A' OR PR.STATUS IS NULL)
      AND 	(T.STATUS = 'A' OR T.STATUS IS NULL)
      AND (T.PERCENT_COMPLETED < 100 OR T.PERCENT_COMPLETED IS NULL)
      GROUP BY AL.RESOURCE_T) AS UNOCCUPANCY_DATE,
      (
        SELECT
        count(1) AS UNOCCUPANCY_DATE
          FROM 		TASK T
          INNER JOIN 	PROJECT PR 			ON (T.PROJECT = PR.ID)
          INNER JOIN 	ALLOCATION AL 		ON (AL.TASK = T.ID)
            WHERE
            AL.RESOURCE_T = R.ID
            AND 	(PR.STATUS = 'A' OR PR.STATUS IS NULL)
            AND 	(T.STATUS = 'A' OR T.STATUS IS NULL)
            AND (T.PERCENT_COMPLETED < 100 OR T.PERCENT_COMPLETED IS NULL)
            GROUP BY AL.RESOURCE_T
        ) AS QUEUED_ACTIVITIES,
        (
        SELECT
        count(1) AS UNOCCUPANCY_DATE
          FROM 		TASK T
          INNER JOIN 	PROJECT PR 			ON (T.PROJECT = PR.ID)
          INNER JOIN 	ALLOCATION AL 		ON (AL.TASK = T.ID)
            WHERE
            AL.RESOURCE_T = R.ID
            AND 	(PR.STATUS = 'A' OR PR.STATUS IS NULL)
            AND 	(T.STATUS = 'A' OR T.STATUS IS NULL)
            AND (T.PERCENT_COMPLETED < 100 OR T.PERCENT_COMPLETED IS NULL)
            GROUP BY AL.RESOURCE_T
        ) AS STATUS,R.CLASS
  FROM RESOURCE_T R) R
  WHERE R.ORGANIZATION = :organizationId
  AND R.ID = :id
SQL;
        $param = array(
			'organizationId' => $organizationId,
            'id' => $resourceID
		);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function getResourceByProject($project, $filter='', $filterAllowedAccessProfile = '') {
		$query = <<<SQL
		SELECT * 
		  FROM ( SELECT R.ID, R.NAME, C.DAYS, C.DAY_HOURS, R.CALENDAR, TO_CHAR(R.RATING, '0.9') AS  RATING
                   FROM RESOURCE_PROJECT RP
              LEFT JOIN RESOURCE_T R 	ON (R.ID = RP.RESOURCE_C)
              LEFT JOIN OPERADOR O ON O.CDOPERADOR = R.EXTERNAL_CODE
              LEFT JOIN PERFACESSPARC PER ON PER.NRPARCNEGOCIO = O.NRPARCNEGOCIO
              LEFT JOIN CALENDAR C 		ON (R.CALENDAR = C.ID)
                  WHERE RP.PROJECT = :project
                    AND R.STATUS = 'A'
                        $filterAllowedAccessProfile
               GROUP BY R.ID, R.NAME, C.DAYS, C.DAY_HOURS, R.CALENDAR ,R.RATING
               ORDER BY R.NAME )
         WHERE 1 = 1 
        $filter 
SQL;
		$param = array(
			'project' => $project
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query,$param);
	}
    public function getTaskResourceMeeting($ids){
		$query = <<<SQL
			SELECT 	R.ID, R.NAME
		 	  FROM 	RESOURCE_T R
			 WHERE  R.ID IN (:id)
               AND  R.STATUS = :isActive
          ORDER BY R.NAME
SQL;
		$param = array(
			'id' => $ids,
			'isActive' => Entities\Project::IS_ACTIVE,

		);
        $paramTypes = array(
            'id' => Connection::PARAM_STR_ARRAY
        );
		return $this->getEntityManager()->getConnection()->fetchAll($query,$param, $paramTypes);
	}

    public function findResourceByStructureId($structureId){
        $query 	= <<<SQL
        SELECT R.ID
        FROM RESOURCE_T R
        JOIN PERMISSION P ON R.ID = P.RESOURCE_T
        WHERE P.STRUCTURE = :structure
        AND (P.TYPE = 'R' OR P.TYPE IS NULL)
        AND R.STATUS = 'A'

SQL;
        $param = array('structure' => $structureId);
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return $result;
    }

	public function getProjectByOrganization($organization){
		$sql= <<<SQL
	   	SELECT DISTINCT R.ID,R.NAME
		FROM RESOURCE_T R
		WHERE R.ORGANIZATION = :organization AND R.CLASS = 'S' AND R.STATUS = 'A'
		ORDER BY R.NAME ASC
SQL;
		$param = array(
			'organization' => $organization
		);
		return $this->getEntityManager()->getConnection()->fetchAll($sql,$param);
	}

	public function getResourceAssetTypes($organizationId){
		$query = <<<SQL
			SELECT P.ID,P.CONTROL,P.LABEL AS NAME
				FROM PROFILE P
				WHERE P.ORGANIZATION = :organizationId
				AND P.CATEGORY = 'A'
SQL;
		$param = array(
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}
	public function getAllEmployee($organizationId,$filterString = ''){
		$query = <<<SQL
		SELECT ID,NAME ,STRUCTURE_NAME, RESOURCE_STATUS FROM (
			  SELECT RES.*,PER.STRUCTURE, STR.NAME AS STRUCTURE_NAME, (CASE WHEN RES.STATUS = 'A' THEN 'Ativo' ELSE 'Inativo' END ) AS RESOURCE_STATUS
				FROM RESOURCE_T RES
           LEFT JOIN PERMISSION PER ON (PER.RESOURCE_T = RES.ID AND NVL(PER.TYPE,'R') = 'R' )
           LEFT JOIN STRUCTURE STR ON (STR.ID = PER.STRUCTURE )
               WHERE RES.ORGANIZATION = :organizationId
				 AND RES.CLASS = 'E')
			   WHERE 1=1
				$filterString
				GROUP BY ID, NAME, STRUCTURE_NAME, RESOURCE_STATUS
				ORDER BY NAME
SQL;
		$param = array(
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}
    public function getProjectManagerIdByEmail($organizationId,$email){
		$query = <<<SQL
			SELECT RES.ID AS "id"
            FROM RESOURCE_T RES
            JOIN PARTNER_PROFILE PPR ON (PPR.RESOURCE_T = RES.ID )
            JOIN PROFILE PRO ON ( PRO.ID = PPR.PROFILE AND PRO.CONTROL = 'M')
            WHERE RES.EMAIL LIKE :email
            AND RES.STATUS = 'A'
            GROUP BY RES.ID
SQL;
		$param = array(
			'organizationId' => $organizationId,
			'email' => $email,
		);
        $managerId = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        if(!empty($managerId)){
            $manager = $this->findOneBy($managerId);
        }else{
            $manager = null;
        }
		return $manager;
	}

	public function getBasicDataResource($resourceId, $organizationId)	{
        $query = <<<SQL
			SELECT 	RES.ID, TO_CHAR(RES.RATING, '0.9') AS  RATING, RES.NAME, RES.EMAIL, RES.PASSWORD, RES.STATUS, RES.DESCRIPTION, RES.RESOURCES_TYPE AS TYPE_CONTROL,
        			PER.ID AS PERMISSION, ST.NAME AS STRUCTURE_NAME, PER.STRUCTURE, P.ID AS ASSET_TYPE, P.LABEL AS ASSET_TYPE_NAME,
        			(TO_CHAR(RES.AVAILABLE_FROM, 'DD/MM/YYYY')) AS AVAILABLE_FROM
			FROM 		RESOURCE_T RES
	        LEFT JOIN 	PERMISSION PER 		ON (PER.RESOURCE_T = RES.ID AND NVL(PER.TYPE,'R') = 'R')
	        LEFT JOIN 	STRUCTURE ST 		ON (ST.ID = PER.STRUCTURE AND ST.STATUS <> 'D')
	        LEFT JOIN 	PARTNER_PROFILE PP 	ON (PP.RESOURCE_T = RES.ID AND RES.RESOURCES_TYPE = 'A')
	        LEFT JOIN 	PROFILE P 			ON (P.ID = PP.PROFILE)
			WHERE 	RES.ID = :resourceId
			AND 	RES.ORGANIZATION = :organizationId
SQL;
		$param = array(
			'resourceId' => $resourceId,
			'organizationId' => $organizationId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function getBusinessPartnerDataResource($resourceId, $organizationId) {
		$query = <<<SQL
			SELECT 	BP.NAME AS NAME, BP.ID AS BP_ID, BP.LOGIN AS EMAIL, BP.PASSWORD, P.ID AS PROFILE,
					P.CONTROL AS PROFILE_NAME, STR.ID AS STRUCTURE, STR.NAME AS STRUCTURE_NAME,
					EMP.DESCRIPTION, CA.CONTROL AS NAME_ENTITY, PER.ID AS PERMISSION, EMP.ID AS EMPLOYEE
				FROM RESOURCE_T RES JOIN BUSINESS_PARTNER BP ON RES.BUSINESS_PARTNER = BP.ID
					LEFT JOIN PARTNER_PROFILE PP ON PP.BUSINESS_PARTNER = BP.ID
					LEFT JOIN PROFILE P ON PP.PROFILE = P.ID
					LEFT JOIN CATEGORY CA ON CA.ID = P.CATEGORY
					LEFT JOIN PERMISSION PER ON BP.ID = PER.BUSINESS_PARTNER
					LEFT JOIN STRUCTURE STR ON PER.STRUCTURE = STR.ID
					LEFT JOIN EMPLOYEE EMP ON EMP.BUSINESS_PARTNER = BP.ID
				WHERE RES.ID = :resourceId AND RES.ORGANIZATION = :organizationId
SQL;
		$param = array(
			'resourceId' => $resourceId,
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function getAssetDataResource($resourceId, $organizationId) {
		$query = <<<SQL
			SELECT 	ASSET.NAME AS NAME, ASSET.ID AS ASSET_ID, ASSET.DESCRIPTION, AT.ID AS ASSET_TYPE,
					AT.NAME AS ASSET_TYPE_NAME, 'Asset' AS NAME_ENTITY
				FROM RESOURCE_T RES
					LEFT JOIN ASSET ASSET ON RES.ASSET = ASSET.ID
					LEFT JOIN ASSET_TYPE AT ON ASSET.ASSET_TYPE = AT.ID
				WHERE RES.ID = :resourceId AND RES.ORGANIZATION = :organizationId
SQL;
		$param = array(
			'resourceId' => $resourceId,
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function getCostDataResource($resourceId, $organizationId) {
		$query = <<<SQL
			SELECT COST.NAME, COST.ID AS COST_ID, COST.DESCRIPTION,'Cost' AS NAME_ENTITY
				FROM RESOURCE_T RES
					LEFT JOIN COST COST ON RES.COST = COST.ID
				WHERE RES.ID = :resourceId AND RES.ORGANIZATION = :organizationId
SQL;
		$param = array(
			'resourceId' => $resourceId,
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function getDataAvailableResourceList($resourceId, $organizationId) {
		$query = <<<SQL
			SELECT avail.ID, TO_CHAR(avail.BEGIN,'DD/MM/YYYY') AS "BEGIN", TO_CHAR(avail.END,'DD/MM/YYYY') AS "END"
				FROM RESOURCE_T res JOIN AVAILABLE avail on res.ID = avail.RESOURCE_T
				WHERE res.ID = :resourceId AND res.ORGANIZATION = :organizationId
SQL;
		$param = array(
			'resourceId' => $resourceId,
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}
	public function getDataCostResourceList($resourceId, $organizationId) {
		$query = <<<SQL
			SELECT  1 AS ID, (CASE WHEN EFFECTIVE_DATE IS NULL THEN 'A' ELSE 'I' END) AS ALL_DAYS,
					(CASE WHEN EFFECTIVE_DATE IS NULL THEN 'Todos os dias' ELSE EFFECTIVE_DATE END) AS EFFECTIVE_DATE,
					SUM(EXTRA_HOUR_TAX) AS EXTRA_HOUR_TAX, SUM(DEFAULT_TAX) AS DEFAULT_TAX, SUM(HOUR_COST) AS HOUR_COST
  				FROM (
    				SELECT 	TO_CHAR(cost.EFFECTIVE_DATE,'DD/MM/YYYY') AS EFFECTIVE_DATE,
				            (CASE WHEN cost.RATE_TYPE = 'E' THEN cost.RATE_VALUE ELSE 0 END) AS EXTRA_HOUR_TAX,
				            (CASE WHEN cost.RATE_TYPE = 'D' THEN cost.RATE_VALUE ELSE 0 END) AS DEFAULT_TAX,
				            (CASE WHEN cost.RATE_TYPE = 'H' THEN cost.RATE_VALUE ELSE 0 END) AS HOUR_COST
					    FROM 		RESOURCE_T res
					    INNER JOIN 	COST_RESOURCE cost on res.ID = cost.RESOURCE_T
					    WHERE 	res.ID = :resourceId
					    AND 	res.ORGANIZATION = :organizationId
  				)
  				GROUP BY EFFECTIVE_DATE
SQL;
		$param = array(
			'resourceId' => $resourceId,
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function getCalendarResource($resourceId, $organizationId) {
		$query= <<<SQL
			SELECT 	CE.ID, C.NAME, C.STATUS, C.BEGIN_WEEK, C.END_WEEK, C.HOUR,
					TO_CHAR(C.BEGIN_HOUR, 'HH24:MI') AS BEGIN_HOUR, TO_CHAR(C.END_HOUR, 'HH24:MI') AS END_HOUR,
					C.WEEKLY_HOURS, C.MONTHLY_DAYS
				FROM CALENDAR_ENTITY CE
					LEFT JOIN CALENDAR C ON (C.CALENDAR_ENTITY = CE.ID)
					LEFT JOIN RESOURCE_T R ON(R.CALENDAR_ENTITY = CE.ID)
				WHERE R.ID = :resourceId AND C.ORGANIZATION = :organizationId
SQL;
		$param = array(
			'resourceId' => $resourceId,
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function getExceptionResource($resourceId, $organizationId) {
		$query = <<<SQL
			SELECT 	CO.ID, CO.CALENDAR, CO.NAME,CO.STATUS, CO.COMMITMENT_TYPE AS TYPE,
					TO_CHAR(CO.BEGIN_DATE,'DD/MM/YYYY') as BEGIN_DATE_LIST, TO_CHAR(CO.END_DATE,'DD/MM/YYYY') as END_DATE_LIST,
					TO_CHAR(CO.BEGIN_DATE,'YYYY-MM-DD') as BEGIN_DATE, TO_CHAR(CO.END_DATE,'YYYY-MM-DD') as END_DATE,
					TO_CHAR(CO.BEGIN_TIME_BREAK,'YYYY-MM-DD') as BEGIN_DATE_BREAK,
					TO_CHAR(CO.END_TIME_BREAK,'YYYY-MM-DD') as END_DATE_BREAK, CO.UNIT, CO.TIME_BREAK
				FROM CALENDAR_ENTITY CE JOIN CALENDAR C ON (C.CALENDAR_ENTITY = CE.ID)
					LEFT JOIN COMMITMENT CO ON (CO.CALENDAR = C.ID)
					LEFT JOIN RESOURCE_T R ON (R.CALENDAR_ENTITY = CE.ID)
				WHERE CE.ORGANIZATION = :organizationId AND R.ID = :resourceId
SQL;
		$param = array(
			'resourceId' => $resourceId,
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function getProfileResourceList($resourceId, $organizationId) {
		$query = <<<SQL
			SELECT P.ID AS PROFILE, P.CONTROL,P.LABEL AS NAME, P.CATEGORY, PP.ID AS ID
				FROM 		PROFILE P
				LEFT JOIN 	PARTNER_PROFILE PP ON PP.PROFILE = P.ID
				LEFT JOIN 	RESOURCE_T R ON R.ID = PP.RESOURCE_T
				WHERE P.ORGANIZATION = :organizationId AND R.ID = :resourceId
SQL;
		$param = array(
			'resourceId' => $resourceId,
			'organizationId' => $organizationId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

	public function findManagerByOrganization($organizationId, $userId, $resourceExternalIds) {
        $resourceExternalIds = implode(",", $resourceExternalIds);
        $sql = <<<SQL
			SELECT DISTINCT R.ID,R.NAME
              FROM RESOURCE_T R
	         INNER JOIN PERMISSION P  ON (P.RESOURCE_T = R.ID)
	         INNER JOIN STRUCTURE S   ON (P.STRUCTURE = S.ID)
	         INNER JOIN PERMISSION PM ON (PM.STRUCTURE = S.ID AND PM.RESOURCE_T = :userId)
             WHERE R.ORGANIZATION = :organizationId
               AND ((PM.TYPE = 'M' ) OR R.ID = :userId)
               AND R.EXTERNAL_ID IN ($resourceExternalIds)
SQL;
		$param = array(
			'organizationId' => $organizationId,
			'userId' => $userId
		);
		$dataSet = $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
		return $dataSet;
	}

	public function findResourcesByTaskId($taskId){
		$sql = <<<SQL
			SELECT A.RESOURCE_T AS ID
					FROM ALLOCATION A
			   WHERE A.TASK = :task
SQL;
		$param = array(
			'task' => $taskId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
	}

	public function findResourcesByOrganization($organizationId) {
		$sql = "SELECT BP.ID, BP.NAME
				FROM RESOURCE_T BP
			   	WHERE BP.ORGANIZATION = :organizationId
                AND BP.RESOURCES_TYPE = 'W'";

		$params = array(
			'organizationId' => $organizationId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
	}
    public function getAllStructureManagers($organizationId,$filters = '') {
		$sql = <<<SQL
		SELECT * FROM (
              SELECT BP.ID, BP.NAME
				FROM RESOURCE_T BP
				JOIN PERMISSION PER ON (PER.RESOURCE_T = BP.ID AND PER.TYPE = 'M')
			   WHERE BP.ORGANIZATION = :organizationId
                 AND BP.RESOURCES_TYPE = 'W'
            GROUP BY BP.ID,BP.NAME
            ORDER BY BP.NAME)
            WHERE 1 = 1
             $filters
SQL;

		$params = array(
			'organizationId' => $organizationId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
	}
    public function getAllManagers($organizationId) {
		$sql = <<<SQL
              SELECT RES.ID, RES.NAME,RES.EMAIL
                FROM RESOURCE_T RES
                JOIN PARTNER_PROFILE PP ON(PP.RESOURCE_T = RES.ID)
                JOIN PROFILE PR ON (PP.PROFILE = PR.ID)
               WHERE PR.CONTROL = 'M'
                 AND res.ORGANIZATION = :organizationId
                 AND RES.STATUS = 'A'
            GROUP BY RES.ID, RES.NAME,RES.EMAIL
            ORDER BY RES.NAME
SQL;

		$params = array(
			'organizationId' => $organizationId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
	}

	public function findResourcesByBusinessPartnerId($businessPartnerId){
		$sql = <<<SQL
			SELECT R.ID
					FROM RESOURCE_T R
			   WHERE R.BUSINESS_PARTNER = :businessPartner
SQL;
		$param = array(
			'businessPartner' => $businessPartnerId
		);

		return $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
	}

    public function getAuthentication($email, $password) {

        return $this->findOneBy(array('isUser'=>'Y','email' => $email,
            'password' => $password, 'status' => 'A'));
    }

    public function getUserByMail($email){
        return $this->findOneBy(array('login' => $email));
    }

    public function getBusinessPartnerByOrganization($organizationId) {
        $sql = "SELECT bp.NAME,bp.ID FROM EMPLOYEE emp
        LEFT JOIN BUSINESS_PARTNER bp ON emp.BUSINESS_PARTNER = bp.ID WHERE emp.ORGANIZATION = :organization";

        $params = array(
            'organization' => $organizationId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
    }

    public function getBusinessPartnerForEditStructure($organizationId) {
        $sql = <<<SQL
        SELECT RES.NAME, RES.ID
        FROM RESOURCE_T RES
        WHERE 	RES.ORGANIZATION = :organization
        AND 	RES.STATUS = 'A'
        AND 	RES.CLASS = 'E'
        GROUP BY RES.NAME, RES.ID
SQL;

        $params = array(
            'organization' => $organizationId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
    }

    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource
     * @return \Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization
     */

    public function getOrganization(\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource){
        return $this->getEntityManager()->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization', $resource->getOrganization());
    }

    public function getSponsorByOrganization($organizationId){

        $query = <<<SQL
			SELECT RE.ID, RE.NAME, RE.STATUS
              FROM RESOURCE_T RE
			 WHERE RE.ORGANIZATION = :organization
               AND RE.CLASS = 'S'
			   AND RE.STATUS <> 'D'
SQL;
        $params = array(
            ":organization" => $organizationId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function basicSponsor($resource){

        $query = <<<SQL
                SELECT RE.ID,RE.NAME,RE.PASSWORD,RE.EMAIL AS LOGIN,RE.NICK_NAME,RE.DOCUMENT_NUMBER,RE.PASSWORD AS PASSWORD_CONFIRM,RE.DESCRIPTION,
                RE.STATUS
                FROM RESOURCE_T RE
                WHERE RE.ID = :resourceT
SQL;
        $params = array(
            ":resourceT" => $resource
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $sponsorId
     * @param $organizationId
     * @return array
     */
    public function getCalendarSponsor($sponsorId, $organizationId)
    {
        $sql = "SELECT CE.ID ,C.NAME ,C.STATUS ,C.BEGIN_WEEK ,C.END_WEEK ,C.HOUR,TO_CHAR(C.BEGIN_HOUR, 'HH24:MI') AS BEGIN_HOUR,TO_CHAR(C.END_HOUR, 'HH24:MI') AS END_HOUR,C.WEEKLY_HOURS,C.MONTHLY_DAYS
                 FROM CALENDAR_ENTITY CE JOIN CALENDAR C ON (C.CALENDAR_ENTITY = CE.ID)
                WHERE CE.ID_TABLE_ENTITY = '_Sponsor_$sponsorId'
                  AND C.ORGANIZATION = :organization";
        $params = array(
            ":organization" => $organizationId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
    }
    /**
     * @param $sponsorId
     * @param $organizationId
     * @return array
     */
    public function getContactSponsor($sponsorId, $organizationId)
    {
        $query = <<<SQL
	        SELECT RES.ID AS CONTACT, RES.NAME, RES.PHONE, RES.EMAIL
            FROM RESOURCE_T RES
            WHERE 	RES.PARENT = :sponsor
            AND 	RES.ORGANIZATION =  :organization
            AND 	RES.STATUS <> 'D'
SQL;
        $params = array(
            ":organization" => $organizationId,
            ":sponsor" => $sponsorId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$params);
    }
    /**
     * @param $sponsor
     * @param $organization
     * @return array
     */
    public function getExceptionSponsor($sponsor, $organization)
    {
        $query = <<<SQL
        SELECT CO.ID,CO.CALENDAR,CO.NAME,CO.STATUS,CO.COMMITMENT_TYPE AS TYPE,
                    TO_CHAR(CO.BEGIN_DATE,'DD/MM/YYYY') as BEGIN_DATE_LIST,
                    TO_CHAR(CO.END_DATE,'DD/MM/YYYY') as END_DATE_LIST,
                    TO_CHAR(CO.BEGIN_DATE,'YYYY-MM-DD') as BEGIN_DATE,
                    TO_CHAR(CO.END_DATE,'YYYY-MM-DD') as END_DATE,
                    TO_CHAR(CO.BEGIN_TIME_BREAK,'YYYY-MM-DD') as BEGIN_DATE_BREAK,
                    TO_CHAR(CO.END_TIME_BREAK,'YYYY-MM-DD') as END_DATE_BREAK,
                    CO.UNIT,CO.TIME_BREAK
                 FROM CALENDAR_ENTITY CE JOIN CALENDAR C ON (C.CALENDAR_ENTITY = CE.ID)
                 JOIN COMMITMENT CO ON(CO.CALENDAR = C.ID)
                WHERE  CE.ORGANIZATION = $organization
                AND CE.ID_TABLE_ENTITY ='_Sponsor_$sponsor'
SQL;
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }
    /**
     * @param $sponsor
     * @param $organization
     * @return array
     */
    public function getSponsorProjectData($sponsorId, $organization, $conditions)
    {
        $conditionExternalType = DevOpsUtilities::queryFilterTypeProjectMeeting('PT', true);
        $query = <<<SQL
            SELECT P.NAME, P.ID, P.PARENT, P.BEGIN_DATE, P.END_DATE, RTRIM(XMLAGG(XMLELEMENT(e, P.MANAGER, ',').EXTRACT('//text()') ORDER BY P.MANAGER).GetClobVal(),',') AS MANAGER,
                   P.PROJECT_STATUS, PROJECT_STATUS_NAME
              FROM (SELECT P.NAME, RG.NAME AS MANAGER, P.ID, P.PROJECT_STATUS, TS.LABEL AS PROJECT_STATUS_NAME,
                           (SELECT MAX(PIN.ID)
                              FROM PROJECT PIN
                              JOIN RESOURCE_PROJECT PPS ON(PIN.ID = PPS.PROJECT AND PPS.TYPE = 'S')
                             WHERE PIN.ORGANIZATION = :ORGANIZATION
                               AND PPS.RESOURCE_C = :SPONSOR
                               AND PIN.STATUS <> 'D'
                               AND PIN.ID = P.PARENT
                           ) AS PARENT, P.BEGIN_DATE, P.END_DATE
                      FROM PROJECT P
                      JOIN RESOURCE_PROJECT PPS ON (P.ID = PPS.PROJECT AND PPS.TYPE = 'S')
                      JOIN RESOURCE_PROJECT PPG ON (P.ID = PPG.PROJECT AND PPG.TYPE = 'M')
                      JOIN RESOURCE_T RG ON RG.ID = PPG.RESOURCE_C
                      JOIN TASK_STATUS TS ON (TS.ID = P.PROJECT_STATUS AND TS.IS_PROJECT = 'Y')
                      LEFT JOIN PROJECT_TYPE PT ON P.PROJECT_TYPE = PT.ID
                     WHERE P.ORGANIZATION = :ORGANIZATION
                       AND PPS.RESOURCE_C = :SPONSOR
                       AND P.STATUS <> 'D'
                $conditionExternalType
                   ) P
             WHERE 1 = 1
       $conditions
		  GROUP BY P.NAME, P.ID, P.PARENT, P.BEGIN_DATE, P.END_DATE, P.PROJECT_STATUS, PROJECT_STATUS_NAME
		  ORDER BY P.ID ASC, P.PARENT ASC, P.BEGIN_DATE DESC, P.END_DATE DESC
            
SQL;
        $param = array(
            'ORGANIZATION' => $organization,
            'SPONSOR' => $sponsorId,
        );
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }
     public function getSponsorActive($organization)
    {
        $query = <<<SQL
            SELECT RE.ID, RE.NAME, RE.STATUS
              FROM RESOURCE_T RE
			 WHERE RE.ORGANIZATION = :organization
               AND RE.CLASS = 'S'
			   AND RE.STATUS = 'A'
SQL;
        $param = array(
            'organization' => $organization
        );
		return $this->getEntityManager()->getConnection()->fetchAll($query,$param);
    }
    /**
     * @param $contact
     * @param $organization
     * @return array
     */
    public function formContact($contact, $organization)
    {
        $query = <<<SQL
              SELECT RE.ID AS CONTACT,RE.NAME,RE.PHONE,RE.STATUS,RE.EMAIL,RE.ADDRESS,RE.DISTRICT,RE.ZIP_CODE,RE.CITY,
              ST.ID AS STATE,CO.ID COUNTRY,ST.NAME AS STATE_NAME,CO.NAME AS COUNTRY_NAME,CI.NAME AS CITY_NAME
                FROM RESOURCE_T RE
                LEFT JOIN CITY CI ON (RE.CITY = CI.ID)
                LEFT JOIN STATE ST ON (CI.STATE = ST.ID)
                LEFT JOIN COUNTRY CO ON (ST.COUNTRY = CO.ID)
                WHERE RE.ORGANIZATION = :ORGANIZATION
                AND RE.ID = :CONTACT
SQL;
        $param = array(
            'ORGANIZATION' => $organization,
            'CONTACT' => $contact,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$param);
    }

    public function formContactList($resource ,$organization, $filters)
    {
        $query = <<<SQL
        SELECT * 
          FROM (SELECT RE.ID AS CONTACT,RE.NAME,RE.PHONE,RE.STATUS,RE.EMAIL,RE.ADDRESS,RE.DISTRICT,RE.ZIP_CODE,RE.CITY,
                        ST.ID AS STATE,CO.ID COUNTRY,ST.NAME AS STATE_NAME,CO.NAME AS COUNTRY_NAME,CI.NAME AS CITY_NAME
                  FROM RESOURCE_T RE
             LEFT JOIN CITY CI ON (RE.CITY = CI.ID)
             LEFT JOIN STATE ST ON (CI.STATE = ST.ID)
             LEFT JOIN COUNTRY CO ON (ST.COUNTRY = CO.ID)
                 WHERE RE.ORGANIZATION = :ORGANIZATION
                   AND RE.PARENT = :sponsor
                   AND RE.STATUS <> 'D'
              ORDER BY RE.ID)
         WHERE 1 = 1
      $filters
SQL;
        $param = array(
            'ORGANIZATION' => $organization,
            'sponsor' => $resource
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$param);
    }
    /**
     * @param $sponsor
     * @param $organization
     * @return array
     */
    public function getAdvancedSponsor($sponsor, $organization)
    {
        $sql = "SELECT
                S.NICK_NAME NICK_NAME,
                S.DOCUMENT_NUMBER DOCUMENT_NUMBER,
                S.DOCUMENT_TYPE DOCUMENT_TYPE,
                S.DESCRIPTION DESCRIPTION
                FROM SPONSOR S JOIN BUSINESS_PARTNER BP ON( S.BUSINESS_PARTNER = BP.ID)
                WHERE S.ID = $sponsor
                AND S.ORGANIZATION = $organization";

        return $this->getEntityManager()->getConnection()->fetchAll($sql);
    }
    /**
     * @param $sponsor
     * @param $organization
     * @return array
     */
    public function getBasicSponsorEdit($sponsor, $organization)
    {
        $sql = "SELECT S.ID ID,
                S.BUSINESS_PARTNER BUSINESS_PARTNER,
                S.NAME NAME,
                S.STATUS STATUS,
                P.ID TYPE,
                P.NAME TYPE_NAME,
                BP.LOGIN LOGIN,
                BP.PASSWORD PASSWORD,
                BP.PASSWORD PASSWORD_CONFIRM
                FROM SPONSOR S JOIN BUSINESS_PARTNER BP ON( S.BUSINESS_PARTNER = BP.ID)
                               JOIN PARTNER_PROFILE PP  ON( PP.BUSINESS_PARTNER = BP.ID)
                               JOIN PROFILE P           ON( P.ID = PP.PROFILE)
                WHERE S.ID = $sponsor
                AND S.ORGANIZATION = $organization";

        return $this->getEntityManager()->getConnection()->fetchAll($sql);
    }

    public function getPaths($projectIds, $resourceId)	{
        $query =  <<<SQL
            SELECT RTRIM(XMLAGG(XMLELEMENT(e, P.ID, ',')       .EXTRACT('//text()') ORDER BY P.ID).GetClobVal(),',')        AS ID,
                   RTRIM(XMLAGG(XMLELEMENT(e, P.IS_MASTER, ',').EXTRACT('//text()') ORDER BY P.IS_MASTER).GetClobVal(),',') AS REPOSITORIES_IS_MASTER,
                   RTRIM(XMLAGG(XMLELEMENT(e, P.PRODUCT_ID, ',').EXTRACT('//text()') ORDER BY P.PRODUCT_ID).GetClobVal(),',') AS PRODUCT_ID,
                   P.NAME, P.DESCRIPTION, P.LOCAL_DIRECTORY AS "dir", P.LINUX_DIRECTORY, P.LOCAL_DIRECTORY, P.URL AS "repository"
              FROM (SELECT R.ID , R.NAME, R.DESCRIPTION, R.URL, R.IS_MASTER,
                           D.LINUX_DIRECTORY, D.LOCAL_DIRECTORY, P.ID AS PRODUCT_ID
                      FROM PROJECT P
                     INNER JOIN REPOSITORY R ON P.ID = R.PROJECT
                     INNER JOIN DIRECTORY D ON R.ID = D.REPOSITORY
                     WHERE D.RESOURCE_T = :resource
                       AND P.ID IN ($projectIds)
                       AND R.STATUS = 'A'
                     ORDER BY LOWER(D.LOCAL_DIRECTORY), LOWER(D.LINUX_DIRECTORY)) P
             GROUP BY P.NAME, P.DESCRIPTION, P.LOCAL_DIRECTORY, P.LINUX_DIRECTORY, P.URL
             ORDER BY LOWER(P.LOCAL_DIRECTORY), LOWER(P.LINUX_DIRECTORY)
SQL;
        $param = array(
            'resource' => $resourceId
        );
        $paths = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($paths as &$path) {
            $path['ID'] =  Utilities::setSelectMultiInQuery($path['ID']);
        }
        return $paths;
    }

    public function validateProjectDirectories($projectIds, $userId = null)	{
        $userquery ='';
        if ($userId){
            $userquery = 'AND D.RESOURCE_T = '.$userId;
        }
        $query =  <<<SQL
            SELECT P.ID AS PROJECT_ID, P.NAME AS PROJECT_NAME,
                   COUNT(D.LOCAL_DIRECTORY) AS COUNT_LOCAL_DIRECTORY,
                   COUNT(D.LINUX_DIRECTORY) AS COUNT_LINUX_DIRECTORY
              FROM PROJECT P
              LEFT JOIN REPOSITORY R ON R.PROJECT = P.ID
              LEFT JOIN DIRECTORY D ON D.REPOSITORY = R.ID $userquery
             WHERE P.ID IN ($projectIds)
             GROUP BY P.ID, P.NAME
SQL;
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }

    public function getDataSetResourcesForProject($organization,$projectId) {
        $query =<<<SQL
           SELECT ID, NAME, TYPE, QUEUED_ACTIVITIES, (
                CASE  WHEN QUEUED_ACTIVITIES = 0 THEN 'Disponível'
                      ELSE 'Em atividade' END
              ) AS STATUS, (
                CASE WHEN QUEUED_ACTIVITIES = 0 THEN 'Agora'
                     ELSE TO_CHAR(UNOCCUPANCY_DATE,'DD/MM/YYYY') END
              ) AS DISPONIBILITY FROM
                (SELECT ID, NAME, TYPE, COUNT(TASK_ID) AS QUEUED_ACTIVITIES, MAX(TASK_ESTIMATED_END_DATE) AS UNOCCUPANCY_DATE
                FROM
                  (SELECT  R.ID AS ID, (
                    CASE WHEN R.BUSINESS_PARTNER IS NOT NULL THEN BP.NAME
                         WHEN R.ASSET IS NOT NULL THEN ASS.NAME
                         ELSE COS.NAME END
                    ) AS NAME, RT.NAME AS TYPE, T.ID AS TASK_ID, T.ESTIMATED_END_DATE AS TASK_ESTIMATED_END_DATE
                  FROM RESOURCE_T R
                    LEFT JOIN BUSINESS_PARTNER BP ON (BP.ID = R.BUSINESS_PARTNER)
                    LEFT JOIN ASSET ASS ON (ASS.ID = R.ASSET)
                    LEFT JOIN COST COS ON (COS.ID = R.COST)
                    LEFT JOIN RESOURCE_TYPE RT ON (R.RESOURCES_TYPE = RT.ID)
                    LEFT JOIN RESOURCE_PROJECT RP ON (R.ID = RP.RESOURCE_C)
                    LEFT JOIN ALLOCATION AL ON (RP.ID = AL.RESOURCE_PROJECT)
                    LEFT JOIN TASK T ON (T.ID = AL.TASK AND T.PERCENT_COMPLETED <> 100)
                  WHERE R.ORGANIZATION = :organization
                  AND   T.STATUS = 'A')
                  GROUP BY ID, NAME, TYPE
                )
SQL;


        $params = array(
            'organization'  => $organization
        );
        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query, $params);
    }

    public function getAllowedProfiles($taskId) {
        $query =<<<SQL
           SELECT PER.NRPERFILACESSO  FROM TASK TK
             JOIN CLASSIFICATION_PERFILACESSO PER ON TK.CLASSIFICATION = PER.CLASSIFICATION
            WHERE TK.ID = :TASK
SQL;


        $params = array(
            'TASK'  => $taskId
        );
        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query, $params);
    }

    public function getDataSetForEditProjectResource($project,$organization, $filterNames = false, $filterOperator = false, $filterValue = false, $filterAllowedAccessProfile = '') {
        $filterString = "WHERE ( ";
        $or = " ";
        if($filterNames) {
            $filterValue = "UPPER('" . $filterValue . "')";
            foreach($filterNames as $filterName) {
                if($filterName == "TYPE"){
                    $filterName = "RESOURCES_TYPE";
                }
                $filterName = "UPPER(" .$filterName . ")";
                $filterString = $filterString." "."$or $filterName $filterOperator $filterValue";
                $or = " OR ";
            }
            $filterString = $filterString.")";
        } else {
            $filterString = "";
        }
        $query = <<<SQL
            SELECT *
              FROM (SELECT DISTINCT R.ID,R.NAME,R.RESOURCES_TYPE,R.STATUS_RESOURCE,RPRO.ID AS ID_BASE
                      FROM (SELECT R.ID, R.NAME, R.RESOURCES_TYPE, R.STATUS AS STATUS_RESOURCE, R.CLASS, R.EXTERNAL_CODE
                              FROM RESOURCE_T R) R
                      JOIN RESOURCE_PROJECT RPRO ON(R.ID = RPRO.RESOURCE_C )
                 LEFT JOIN OPERADOR O ON O.CDOPERADOR = R.EXTERNAL_CODE
                 LEFT JOIN PERFACESSPARC PER ON PER.NRPARCNEGOCIO = O.NRPARCNEGOCIO
                     WHERE RPRO.PROJECT = :project
                       AND R.STATUS_RESOURCE <> 'D'
                       AND (R.RESOURCES_TYPE = 'W' OR R.RESOURCES_TYPE = 'A')
                  $filterAllowedAccessProfile
                       )
            $filterString
SQL;
        $params = array(
            'organization' => $organization,
            'project' => $project
        );
        $connection = $this->getEntityManager()->getConnection();
        return $connection->fetchAll($query, $params);
    }

    public function getResourcesByProfile($organizationId, $profileId){

		$query = <<<SQL
			SELECT 	R.ID, R.NAME
			FROM 		RESOURCE_T R
			LEFT JOIN 	PARTNER_PROFILE PP 	ON (R.ID = PP.RESOURCE_T)
			LEFT JOIN 	PROFILE P 			ON (PP.PROFILE = P.ID)
			WHERE 	P.ID = :profileId
			AND 	R.STATUS <> 'D'
SQL;
		$param = array(
			'organizationId' => $organizationId,
			'profileId' => $profileId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);

	}

	public function getUsersAgainstProfile($profileId, $organizationId){
		$query = <<<SQL
			SELECT 	R.ID, R.NAME
			FROM 		RESOURCE_T R
			LEFT JOIN 	PARTNER_PROFILE PP ON R.ID = PP.RESOURCE_T
			WHERE 	PP.PROFILE <> :profileId OR PP.ID IS NULL
			AND 	R.STATUS <> 'D'
			AND 	R.ORGANIZATION = :organizationId
			AND 	R.IS_USER = 'Y'
			GROUP BY R.ID, R.NAME
SQL;
		$param = array(
			'organizationId' => $organizationId,
			'profileId' => $profileId
		);
		return $this->getEntityManager()->getConnection()->fetchAll($query, $param);

	}
    public function getExternalUserId($resourceId){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource */
        $resource = $this->find($resourceId);
		return array(['EXTERNAL_ID' => $resource->getExternalId()]);
	}
    public function findManagerProjectEdit($organizationId, $resourceExternalIds) {
        $resourceExternalIds = implode(",", $resourceExternalIds);
        $filter = "";
        if(!empty($resourceExternalIds)){
            $filter = " AND R.EXTERNAL_ID IN ($resourceExternalIds)";
        }
        $sql = <<<SQL
			SELECT  R.ID,R.NAME
              FROM RESOURCE_T R
	          JOIN PERMISSION P	ON (P.RESOURCE_T = R.ID)
             WHERE R.ORGANIZATION = :organizationId
               AND R.STATUS = 'A'
               $filter
          GROUP BY R.ID,R.NAME
          ORDER BY R.NAME

SQL;

        $param = array(
            'organizationId' => $organizationId

        );
        $dataSet = $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
        return $dataSet;
    }

    public function findManagerProjectEditByName($organizationId, $resourceExternalIds, $filter) {
        $resourceExternalIds = implode(",", $resourceExternalIds);
        $externalCodeString = '';
        if(!empty($resourceExternalIds)){
            $externalCodeString= " AND R.EXTERNAL_ID IN ($resourceExternalIds)";
        }
        $sql = <<<SQL
			SELECT * FROM(
                SELECT DISTINCT R.ID,R.NAME
                  FROM RESOURCE_T R
                  JOIN PERMISSION P	ON (P.RESOURCE_T = R.ID)
                 WHERE R.ORGANIZATION = :organizationId
                   AND R.STATUS = 'A'
                   $externalCodeString
             )
             WHERE 1 =1
             $filter

SQL;

        $param = array(
            'organizationId' => $organizationId

        );
        $dataSet = $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
        return $dataSet;
    }


    public function findManagerProjectEditFilter($organizationId, $param, $resourceExternalIds) {
        $values = $param[0]['value'];
        $iDs = [];
        foreach ($values as $value) {
        	array_push($iDs, intval($value));
        }
        $parentString = implode(",", $iDs);
        $resourceExternalIds = implode(",", $resourceExternalIds);
        $sql = <<<SQL
			SELECT DISTINCT R.ID,R.NAME
            FROM    RESOURCE_T R
	        JOIN 	PERMISSION P 		ON (P.RESOURCE_T = R.ID)
	        JOIN 	STRUCTURE S 		ON (P.STRUCTURE = S.ID AND S.ID IN ($parentString))
           WHERE 	R.ORGANIZATION = :organizationId
            AND     R.STATUS = 'A'
               AND R.EXTERNAL_ID IN ($resourceExternalIds)
SQL;
        $param = array(
            'organizationId' => $organizationId

        );
        $dataSet = $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
        return $dataSet;
    }

    public function findStructuresManagers($structuresIds, $organizationId, $resourceExternalIds) {
        $parentString = implode(",", $structuresIds);
        $resourceExternalIds = implode(",", $resourceExternalIds);

        $sql = <<<SQL
			SELECT DISTINCT R.ID,R.NAME
              FROM RESOURCE_T R
	          JOIN PERMISSION P 		ON (P.RESOURCE_T = R.ID)
	          JOIN STRUCTURE S 		ON (P.STRUCTURE = S.ID AND S.ID IN ($parentString))
             WHERE R.ORGANIZATION = :organizationId
               AND R.STATUS = 'A'
               AND R.EXTERNAL_ID IN ($resourceExternalIds)
SQL;
        $param = array(
            'organizationId' => $organizationId

        );
        $dataSet = $this->getEntityManager()->getConnection()->fetchAll($sql, $param);
        return $dataSet;
    }

	public function findStructureByResourceId($resourceId){
		$query 	= <<<SQL
		SELECT DISTINCT		S.ID, S.NAME
		FROM	PARTNER_PROFILE PP
		JOIN	STRUCTURE S		ON (S.ID = PP.PROFILE)
		JOIN	RESOURCE_T R 	ON (PP.RESOURCE_T = :resourceId)
SQL;
		$param = array('resourceId' => $resourceId);
		$result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
		return $result;
	}

	public function findResourceManagerProfilesByOrganization($organizationId,$filter = ''){
		$query 	= <<<SQL
		SELECT * FROM (
		SELECT RES.ID,RES.NAME
            FROM RESOURCE_T RES
            JOIN OPERADOR OPE ON(OPE.CDOPERADOR = RES.EXTERNAL_CODE)
            JOIN PERFACESSPARC PEPA ON(OPE.NRPARCNEGOCIO = PEPA.NRPARCNEGOCIO)
            JOIN PERFILACESSO PERF ON (PERF.NRPERFILACESSO = PEPA.NRPERFILACESSO AND PERF.NRORG = PEPA.NRORG)
            JOIN PROFILE PR ON (PERF.NRPERFILACESSO = PR.EXTERNAL_ID AND PERF.NRORG = PR.ORGANIZATION)
           WHERE  PR.CONTROL = 'M' AND RES.STATUS = 'A'
           AND PR.ORGANIZATION = :organizationId)
           WHERE 1 = 1
            $filter
            GROUP BY ID,NAME
            ORDER BY NAME
SQL;
		$param = array('organizationId' => $organizationId);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}

    public function findResourceSponsor($resourceId, $filters){
        $query = <<<SQL
        SELECT *
          FROM ( SELECT ID, NAME, STATUS, ORGANIZATION, CLASS
                   FROM RESOURCE_T
                  WHERE CLASS = :CLASS
                    AND STATUS <> 'D' )
		 WHERE 1 = 1
		   $filters
SQL;
        $param = array('CLASS' => Entities\Resource::CLASS_SPONSOR);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }
    
    public function findSponsor($nrOrg, $filterString, $limit, $offset)
    {
        $query = <<<SQL
         SELECT ID, NAME
            FROM RESOURCE_T
          WHERE CLASS = 'S' AND ORGANIZATION = :nrOrg
           $filterString
            AND STATUS = 'A'
SQL;
        $params = array(
		    'nrOrg' => $nrOrg
		);

        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }    
    
    public function findProjectByUser($userId, $date){
        
        $query = <<<SQL
         SELECT  P.ID, 'P'||P.ID AS ID_TREE, P.NAME, '' AS PROJECT_TREE
        FROM PROJECT P
        WHERE P.ID IN (
            SELECT T.PROJECT
            FROM TASK T
            WHERE T.ID IN(
                SELECT A.TASK
                FROM APPROPRIATION A
                WHERE A.RESOURCE_T = :userId
                GROUP BY A.TASK)
        )
        UNION
        SELECT T.ID , 'T'||T.ID AS ID_TREE, T.NAME, 'P'||T.PROJECT AS PROJECT_TREE
                FROM TASK T
                WHERE T.ID IN(
                    SELECT A.TASK
                    FROM APPROPRIATION A
                    WHERE A.RESOURCE_T = :userId
                    GROUP BY A.TASK)
SQL;

		$params = array(
		    'userId' => $userId
		);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findManagerByResourceId($resourceId)
    {
        $query = <<<SQL
         SELECT PE.RESOURCE_T AS MANAGER
           FROM PERMISSION P
           JOIN PERMISSION PE ON (PE.STRUCTURE = P.STRUCTURE AND PE.TYPE = :TYPE_MANAGER)
          WHERE P.RESOURCE_T = :RESOURCE_ID 
            AND (P.TYPE IS NULL OR P.TYPE = :TYPE_RESOURCE)
SQL;
        $params = array(
            'RESOURCE_ID' => $resourceId,
            'TYPE_MANAGER' => Entities\Permission::TYPE_MANAGER,
            'TYPE_RESOURCE' => Entities\Permission::TYPE_RESOURCE
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findOrganizationByResourceId($resourceId)
    {
        $query = <<<SQL
          SELECT O.NRORG
            FROM RESOURCE_T RT
            JOIN CLIENTE C ON C.CDCLIENTE = RT.EXTERNAL_CODE
            JOIN ORGANIZACAO O ON O.NRPARCNEGOCIO = C.NRPARCNEGOCIO
           WHERE RT.ID = :RESOURCE_ID
             AND RT.CLASS = :CLASS_SPONSOR
SQL;
        $params = array(
            'RESOURCE_ID' => $resourceId,
            'CLASS_SPONSOR' => Entities\Resource::CLASS_SPONSOR
        );

        $nrOrg = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return !empty($nrOrg) ? $nrOrg[0] : $nrOrg;
    }

}
